import os
from typing import Any, Dict, List, Literal, Optional, Union

import pyterrier as pt
import pyterrier_alpha as pta

import pyterrier_anserini
from pyterrier_anserini import J
from pyterrier_anserini._similarity import DEFAULT_WMODEL_ARGS, AnseriniSimilarity

_TFields = Union[List[str], str, Literal['*']]

@pt.java.required
class AnseriniIndex(pta.Artifact):
    """An Anserini index.

    An Anserini index is a directory containing a Lucene index built with Anserini.

    This object can be used to construct retrieval transformers.
    """

    def __init__(self, path: str):
        """Initializes a new Anserini index.

        Args:
            path: The path to the index.
        """
        self.path = path

    def built(self) -> bool:
        """Checks if this index is built.

        Returns:
            True if this index is built, False otherwise.
        """
        return os.path.exists(self.path)

    def indexer(self,
        *,
        fields: _TFields = '*',
        verbose: bool = False
    ) -> pt.Indexer:
        """Provides an indexer for this index.

        Args:
            fields: The fields to index. If '*' (default), all fields are indexed. Otherwise, the values of the
            fields provided in this argument are concatenated and indexed.
            verbose: Whether to display a progress bar when indexing.
        """
        return pyterrier_anserini.AnseriniIndexer(self,
            fields=fields,
            verbose=verbose)

    def retriever(self,
        similarity: Union[str, AnseriniSimilarity],
        similarity_args: Optional[Dict[str, Any]] = None,
        *,
        num_results: int = 1000,
        include_fields: Optional[_TFields] = None,
        verbose: bool = False
    ) -> pt.Transformer:
        """Provides a retriever that uses the specified similarity function.

        Args:
            similarity: The similarity function to use.
            similarity_args: The arguments to the similarity function. Defaults to None (no arguments).
            num_results: The number of results to return. Defaults to 1000.
            include_fields: A list of the fields to include in the results. If `None` (default), no extra fields are
            included. If '*', all fields are included.
            verbose: Output verbose logging. Defaults to False.

        Returns:
            A transformer that can be used to retrieve documents from this index.
        """
        return pyterrier_anserini.AnseriniRetriever(
            index=self,
            similarity=similarity,
            similarity_args=similarity_args,
            num_results=num_results,
            include_fields=self._resolve_fields(include_fields),
            verbose=verbose)

    def bm25(self,
        *,
        k1: float = DEFAULT_WMODEL_ARGS['bm25.k1'],
        b: float = DEFAULT_WMODEL_ARGS['bm25.b'],
        num_results: int = 1000,
        include_fields: Optional[_TFields] = None,
        verbose: bool = False
    ) -> pt.Transformer:
        """Providers a retriever that uses BM25 over this index.

        Args:
            k1: The BM25 k1 parameter. Defaults to 0.9.
            b: The BM25 b parameter. Defaults to 0.4.
            num_results: The number of results to return. Defaults to 1000.
            include_fields: A list of the fields to include in the results. If `None` (default), no extra fields are
            included. If '*', all fields are included.
            verbose: Output verbose logging. Defaults to False.

        Returns:
            A transformer that can be used to retrieve documents from this index using qld.
        """
        return pyterrier_anserini.AnseriniRetriever(
            index=self,
            similarity=AnseriniSimilarity.bm25,
            similarity_args={'bm25.k1': k1, 'bm25.b': b},
            num_results=num_results,
            include_fields=self._resolve_fields(include_fields),
            verbose=verbose)

    def qld(self,
        *,
        mu: float = DEFAULT_WMODEL_ARGS['qld.mu'],
        num_results: int = 1000,
        include_fields: Optional[_TFields] = None,
        verbose: bool = False
    ) -> pt.Transformer:
        """Providers a retriever that uses Query Likelihood with Dirichlet smoothing over this index.

        Args:
            mu: The Dirichlet smoothing parameter. Defaults to 1000.
            num_results: The number of results to return. Defaults to 1000.
            include_fields: A list of the fields to include in the results. If `None` (default), no extra fields are
            included. If '*', all fields are included.
            verbose: Output verbose logging. Defaults to False.

        Returns:
            A transformer that can be used to retrieve documents from this index using qld.
        """
        return pyterrier_anserini.AnseriniRetriever(
            index=self,
            similarity=AnseriniSimilarity.qld,
            similarity_args={'qld_mu': mu},
            num_results=num_results,
            include_fields=self._resolve_fields(include_fields),
            verbose=verbose)

    def tfidf(self,
        *,
        num_results: int = 1000,
        include_fields: Optional[_TFields] = None,
        verbose: bool = False
    ) -> pt.Transformer:
        """Provides a TF-IDF retriever over this index.

        Args:
            num_results: The number of results to return. Defaults to 1000.
            include_fields: A list of the fields to include in the results. If `None` (default), no extra fields are
            included. If '*', all fields are included.
            verbose: Output verbose logging. Defaults to False.

        Returns:
            A transformer that can be used to retrieve documents from this index using TF-IDF.
        """
        return pyterrier_anserini.AnseriniRetriever(
            index=self,
            similarity=AnseriniSimilarity.tfidf,
            num_results=num_results,
            include_fields=self._resolve_fields(include_fields),
            verbose=verbose)

    def impact(self,
        *,
        num_results: int = 1000,
        include_fields: Optional[_TFields] = None,
        verbose: bool = False
    ) -> pt.Transformer:
        """Provides a retriever for pre-comptued impact scores.

        This is called "quantized" scoring in PISA or "TF" scoring in Terrier.

        Args:
            num_results: The number of results to return. Defaults to 1000.
            include_fields: A list of the fields to include in the results. If `None` (default), no extra fields are
            included. If '*', all fields are included.
            verbose: Output verbose logging. Defaults to False.

        Returns:
            A transformer that can be used to retrieve documents from this index using impact scores.
        """
        return pyterrier_anserini.AnseriniRetriever(
            index=self,
            similarity=AnseriniSimilarity.impact,
            num_results=num_results,
            include_fields=self._resolve_fields(include_fields),
            verbose=verbose)

    def reranker(self,
        similarity: Union[str, AnseriniSimilarity],
        similarity_args: Optional[Dict[str, Any]] = None,
        *,
        verbose: bool = False
    ) -> pt.Transformer:
        """Provides a reranker that uses the specified weithing model.

        Args:
            similarity: The similarity function to use.
            similarity_args: The arguments to the similarity function. Defaults to None (no arguments).
            verbose: Output verbose logging. Defaults to False.

        Returns:
            A transformer that can be used to score (rerank) documents from this index.
        """
        return pyterrier_anserini.AnseriniReRanker(
            index=self,
            similarity=similarity,
            similarity_args=similarity_args,
            verbose=verbose)

    def text_loader(self,
        fields: Union[List[str], str, Literal['*']] = '*',
        *,
        verbose: bool = False,
    ) -> pt.Transformer:
        """Provides a transformer that can be used to load the text from this index for each document.

        Args:
            fields: The fields to extract. When the literal '*' (default), extracts all available fields.
            verbose: Output verbose logging. Defaults to False.

        Returns:
            A transformer that can be used to load the text from this index for each document.
        """
        return pyterrier_anserini.AnseriniTextLoader(
            index=self,
            fields=self._resolve_fields(fields),
            verbose=verbose)

    def _searcher(self):
        from pyserini.search.lucene import LuceneSearcher
        assert self.built(), "a searcher object can only be created if the index is built"
        return LuceneSearcher(self.path)

    def fields(self) -> List[str]:
        field_info = J.IndexReaderUtils.getFieldInfo(self._searcher().object.reader)
        return [k for k in field_info if k != 'id']

    def _resolve_fields(self, fields: Optional[_TFields]) -> Optional[List[str]]:
        if fields is None:
            return None
        if fields == '*':
            return self.fields()
        if isinstance(fields, str):
            return [fields]
        return fields

    def num_docs(self) -> int:
        return self._searcher().object.get_total_num_docs()

    def __repr__(self):
        return f"AnseriniIndex({self.path!r})"



from typing import Dict, Union

import pandas as pd
import pyterrier as pt
import pyterrier_alpha as pta

from pyterrier_anserini import J
from pyterrier_anserini._index import AnseriniIndex
from pyterrier_anserini._similarity import AnseriniSimilarity


@pt.java.required
class AnseriniReRanker(pt.Transformer):
    """A transformer that scores (i.e., re-ranks) the provided documents from an Anserini index."""
    def __init__(self,
        index: Union[AnseriniIndex, str],
        similarity: Union[str, AnseriniSimilarity],
        similarity_args: Dict = None,
        *,
        verbose: bool = False
    ):
        """Initializes the scorer.

        Args:
            index: The index to score from. If a string, an AnseriniIndex object is created for the path.
            similarity: The similarity function to use for scoring.
            similarity_args: A dictionary of arguments to use for the similarity function.
            verbose: Whether to display a progress bar when scoring.
        """
        self.index = index if isinstance(index, AnseriniIndex) else AnseriniIndex(index)
        self.similarity = AnseriniSimilarity(similarity)
        self.similarity_args = similarity_args
        self.verbose = verbose

    __repr__ = pta.transformer_repr

    def transform(self, inp: pd.DataFrame) -> pd.DataFrame:
        """Scores (i.e., re-ranks) documents from the index for each query in `inp`.

        Args:
            inp: A DataFrame with a 'query' column containing queries and a 'docno' column containing document IDs.

        Returns:
            A DataFrame containing the scored documents, with any columns included in `inp`, plus
            the 'score' and 'rank' of the scored documents.
        """
        with pta.validate.any(inp) as v:
            v.result_frame(['query_lucene'], mode='query_lucene')
            v.result_frame(['query_toks'], mode='query_toks')
            v.result_frame(['query'], mode='query_text')

        sim = AnseriniSimilarity(self.similarity).to_lucene_sim(self.similarity_args)
        index_reader = self.index._searcher().object.reader

        if v.mode == 'query_lucene':
            raise NotImplementedError('query_lucene not yet supported for AnseriniReRanker')
        elif v.mode == 'query_toks':
            raise NotImplementedError('query_toks not yet supported for AnseriniReRanker')
        elif v.mode == 'query_text':
            it = zip(inp['docno'], inp['query'])

        if self.verbose:
            it = pt.tqdm(it, unit='d', total=len(inp), desc='AnseriniScorer')

        scores = [
            J.IndexReaderUtils.computeQueryDocumentScoreWithSimilarity(index_reader, docno, query, sim)
            for docno, query in it
        ]
        res = inp.assign(score=scores)

        return pt.model.add_ranks(res)




from typing import List, Union

import pandas as pd
import pyterrier as pt
import pyterrier_alpha as pta

import pyterrier_anserini
from pyterrier_anserini import AnseriniIndex


@pt.java.required
class AnseriniTextLoader(pt.Transformer):
    """A transformer that provides access to text fields from an Anserini index."""
    def __init__(self,
                 index: Union[AnseriniIndex, str],
                 fields: List[str],
                 *,
                 verbose: bool = False):
        """Initializes the text loader.

        Args:
            index: The index to provide text from. If a string, an AnseriniIndex object is created for the path.
            fields: The fields to load.
            verbose: Whether to display a progress bar when providing text.
        """
        self.index = index if isinstance(index, AnseriniIndex) else AnseriniIndex(index)
        self.fields = fields
        self.verbose = verbose

    __repr__ = pta.transformer_repr

    def transform(self, inp: pd.DataFrame) -> pd.DataFrame:
        """Provides text from the index for each document in `inp`.

        Args:
            inp: A DataFrame with a 'docno' column containing document IDs.
        """
        pta.validate.columns(inp, includes=['docno'])

        utils = pyterrier_anserini.J.IndexReaderUtils
        index_reader = self.index._searcher().object.reader

        results = pta.DataFrameBuilder(['_index'] + self.fields)

        it = enumerate(inp['docno'])
        if self.verbose:
            it = pt.tqdm(it, unit='d', total=len(inp), desc='AnseriniTextLoader')

        for i, docno in it:
            doc = utils.document(index_reader, docno)
            res = {f: doc.get(f) for f in self.fields}
            res['_index'] = i
            results.extend(res)

        return results.to_df(merge_on_index=inp)



import unittest

import pyterrier as pt
import pyterrier_alpha as pta


class TestAnseriniIndex(unittest.TestCase):
    def test_load_from_anserini_url(self):
        # This tests a few things: that the anserini: parser is working, that the metadata adapter works, etc.
        index = pta.Artifact.from_url('anserini:beir-v1.0.0-scifact.flat')
        self.assertEqual(5183, index.num_docs())



import pandas as pd
import pyterrier as pt
import pyterrier_alpha as pta
from deprecated import deprecated

import pyterrier_anserini


@deprecated(version='0.0.1', reason='Use AnseriniRetriever or AnseriniReRanker insead')
@pt.java.required
class AnseriniBatchRetrieve(pt.Transformer):
    """A backwards-compatible interface for Anserini retrieval and scoring."""

    def __init__(self, index_location: str, k: int = 1000, similarity: str = "BM25", verbose: bool = False):
        """Construct an AnseriniBatchRetrieve retrieve from pyserini.search.lucene.LuceneSearcher.

        Args:
            index_location: The location of the Anserini index.
            k: number of results to return. Default is 1000.
            similarity: Similarity model supported by Anserini. There are three options:
             * `"BM25"` - the BM25 similarity function
             * `"QLD"`  - Dirichlet language modelling
             *  `"TFIDF"` - Lucene's `ClassicSimilarity <https://lucene.apache.org/core/8_1_0/core/org/apache/lucene/search/similarities/ClassicSimilarity.html>`_.
            verbose: If True, print verbose output. Default is False.
            kwargs: ignored (for backwards compatibility)
        """
        self.index_location = index_location
        self.k = k
        self.similarity = similarity
        self.verbose = verbose

    __repr__ = pta.transformer_repr

    def transform(self, inp: pd.DataFrame) -> pd.DataFrame:
        """Retrieves or scores the values in `inp`.

        When `inp` is a query frame, retrieves documents from the index for each query in `inp`.

        When `inp` is a result set, scores the documents in the result set.
        """
        with pta.validate.any(inp) as v:
            v.query_frame(['query'], mode='retrieve')
            v.result_frame(['query'], mode='rerank')

        if v.mode == 'rerank':
            transformer = pyterrier_anserini.AnseriniReRanker(
                self.index_location,
                similarity=self.similarity,
                verbose=self.verbose)

        elif v.mode == 'retrieve':
            transformer = pyterrier_anserini.AnseriniRetriever(
                self.index_location,
                similarity=self.similarity,
                num_results=self.k,
                verbose=self.verbose)

        return transformer(inp)



from typing import Any, Dict, List, Optional
from urllib.parse import ParseResult

import pyterrier as pt


@pt.java.required
def _anserini_url_resolver(parsed_url: ParseResult) -> str:
    """Resolve URLs like Artifact.from_url("anserini:some-index")."""
    from pyserini.util import download_prebuilt_index
    return download_prebuilt_index(parsed_url.path)


def _anserini_metadata_adapter(path: str, dir_listing: List[str]) -> Optional[Dict[str, Any]]:
    """Guess whether this path is an anserini index.

    There are a few files that seem to be a good indicator. Check for those.

    (Actually, these identify any lucene index, but I guess this is alright.)
    """
    if 'write.lock' in dir_listing and any(x.startswith('segments_') for x in dir_listing):
        return {
            'type': 'sparse_index',
            'format': 'anserini',
            'package_hint': 'pyterrier-anserini',
        }



import importlib.metadata
import os
from glob import glob
from pathlib import Path
from typing import Optional, Tuple
from warnings import warn

import pyterrier as pt
from packaging.version import Version

configure = pt.java.register_config('pyterrier.anserini', {
    'version': None,
})
_version = None


class AnseriniJavaInit(pt.java.JavaInitializer):
    def __init__(self):
        self._message = None

    def condition(self) -> bool:
        """Disables loading with anserini >= 0.36 since it introduces incompatible dependencies."""
        try:
            importlib.metadata.version('pyserini')
        except Exception as ex:
            warn(f'error loading anserini java: {ex}')
            return False
        return True

    def pre_init(self, jnius_config): # noqa: ANN001
        global _version
        if configure['version'] is None:
            jar, version = _get_pyserini_jar()
            self._message = f"version={version} (from pyserini package)"
            _version = version
        else:
            # download and use the anserini version specified by the user
            jar = pt.java.mavenresolver.get_package_jar(
                'io.anserini', "anserini", configure['version'], artifact='fatjar')
            self._message = f"version={configure['version']} (local cache)"
            _version = configure['version']

        if jar is None:
            raise RuntimeError('Could not find anserini jar')
        else:
            jnius_config.add_classpath(jar)

    def post_init(self, jnius): # noqa: ANN001
        # Temporarily disable the configure_classpath during pyserini init, otherwise it will try to reconfigure jnius
        import pyserini.setup
        _configure_classpath = pyserini.setup.configure_classpath
        try:
            pyserini.setup.configure_classpath = pt.utils.noop
            import pyserini.search.lucene  # load the package
        finally:
            pyserini.setup.configure_classpath = _configure_classpath

    def message(self):
        return self._message


def _get_pyserini_jar() -> Optional[Tuple[str, str]]:
    # find the anserini jar distributed with pyserini
    # Adapted from pyserini/setup.py and pyserini/pyclass.py
    import pyserini.setup
    jar_root = os.path.join(os.path.split(pyserini.setup.__file__)[0], 'resources/jars/')
    paths = glob(os.path.join(jar_root, 'anserini-*-fatjar.jar'))
    if not paths:
        return None, None
    latest_jar = max(paths, key=os.path.getctime)
    version = Path(latest_jar).name.split('-')[-2]
    return latest_jar, version


@pt.java.before_init
def set_version(version: Optional[str] = None):
    """Set the version of Anserini to use.

    If version is ``None`` (default), the version of Anserini distributed with the pyserini package is used. Otherwise,
    the specified version is downloaded from Maven and used insead.

    Note that this function must be run before Java is initialized.
    """
    configure['version'] = version


@pt.java.required
def check_version(min_version: str) -> bool:
    return Version(min_version) <= Version(_version)


J = pt.java.JavaClasses(
    ClassicSimilarity = 'org.apache.lucene.search.similarities.ClassicSimilarity',
    BM25Similarity = 'org.apache.lucene.search.similarities.BM25Similarity',
    LMDirichletSimilarity = 'org.apache.lucene.search.similarities.LMDirichletSimilarity',
    IndexReaderUtils = 'io.anserini.index.IndexReaderUtils',
    QueryParser = 'org.apache.lucene.queryparser.classic.QueryParser',
    ImpactSimilarity = 'io.anserini.search.similarity.ImpactSimilarity',
)



import json
import os
from typing import Dict, Iterable, List, Literal, Union

import pyterrier as pt
import pyterrier_alpha as pta

from pyterrier_anserini import AnseriniIndex


@pt.java.required
class AnseriniIndexer(pt.Indexer):
    """An indexer for Anserini indexes."""
    def __init__(self,
        index: Union[AnseriniIndex, str],
        *,
        fields: Union[List[str], Literal['*']] = '*',
        verbose: bool = False
    ):
        """Initializes the indexer.

        Args:
            index: The index to index to. If a string, an AnseriniIndex object is created for the path.
            fields: The fields to index. If '*' (default), all fields are indexed. Otherwise, the values of the fields
                provided in this argumetn are concatenated and indexed.
            verbose: Whether to display a progress bar when indexing.
        """
        self._index = index if isinstance(index, AnseriniIndex) else AnseriniIndex(index)
        self.fields = fields
        self.verbose = verbose

    __repr__ = pta.transformer_repr

    def index(self, inp: Iterable[Dict]) -> pta.Artifact:
        """Indexes the input documents to the index.

        Args:
            inp: An iterable of documents to index.

        Returns:
            The index that was indexed to.
        """
        assert not self._index.built()
        from pyserini.index.lucene import LuceneIndexer
        args = ['-index', self._index.path, '-storeContents', '-storeDocvectors']
        indexer = LuceneIndexer(self._index.path, args=args)
        # create directory and metadata file
        if not os.path.exists(os.path.join(self._index.path, 'pt_meta.json')):
            os.makedirs(self._index.path, exist_ok=True)
            with open(os.path.join(self._index.path, 'pt_meta.json'), 'wt') as fout:
                json.dump({
                    'type': 'sparse_index',
                    'format': 'anserini',
                    'package_hint': 'pyterrier-anserini',
                    # TODO: other stuff (like stemmer used) in due course
                }, fout)

        if self.verbose:
            inp = pt.tqdm(inp, unit='docs', desc='AnseriniIndexer')

        for doc in inp:
            indexer.add_doc_dict(self._map_doc(doc))

        # commit
        indexer.close()

        return self._index

    def _map_doc(self, doc: Dict) -> Dict:
        if self.fields == '*':
            contents = '\n'.join(v for k, v in doc.items() if k != 'docno' and isinstance(v, str))
        else:
            contents = '\n'.join(str(doc[k]) for k in self.fields)
        return {
            'id': doc['docno'],
            'contents': contents
        }



from typing import Any, Dict, List, Optional, Union

import numpy as np
import pandas as pd
import pyterrier as pt
import pyterrier_alpha as pta

from pyterrier_anserini import J
from pyterrier_anserini._index import AnseriniIndex
from pyterrier_anserini._similarity import AnseriniSimilarity


def _noop_query_parser(query: str) -> str:
    return query


def _toks_query_parser_factory(parser): # noqa: ANN001
    def wrapped(toks: Dict[str, float]) -> Any:
        res = []
        for tok, weight in toks.items():
            res.append(f'{parser.escape(tok)}^{weight:f}')
        query = ' '.join(res)
        return parser.parse(query)
    return wrapped


@pt.java.required
class AnseriniRetriever(pt.Transformer):
    """Retrieves from an Anserini index."""
    def __init__(self,
        index: Union[AnseriniIndex, str],
        similarity: Union[AnseriniSimilarity, str] = "BM25",
        similarity_args: Dict[str, any] = None,
        *,
        num_results: int = 1000,
        include_fields: Optional[List[str]] = None,
        verbose: bool = False,
    ):
        """Construct an AnseriniRetriever retrieve from pyserini.search.lucene.LuceneSearcher.

        Args:
            index: The Anserini index.
            similarity: The similarity function to use.
            similarity_args: model-specific arguments, like bm25.k1.
            num_results: number of results to return. Default is 1000.
            include_fields: a list of extra stored fields to include for each result. `None` indicates no extra fields.
            verbose: show a progress bar during retrieval?
        """
        if not isinstance(index, AnseriniIndex):
            index = AnseriniIndex(index)
        self.index = index
        self.similarity = similarity
        self.similarity_args = similarity_args
        self.num_results = num_results
        self.include_fields = include_fields
        self.verbose = verbose

    __repr__ = pta.transformer_repr

    def transform(self, inp: pd.DataFrame) -> pd.DataFrame:
        """Performs retrieval.

        Args:
            inp: A pandas.Dataframe

        Returns:
            pandas.DataFrame with columns=['qid', 'query', 'docno', 'rank', 'score']
        """
        with pta.validate.any(inp) as v:
            v.query_frame(extra_columns=['query_lucene'], mode='query_lucene')
            v.query_frame(extra_columns=['query_toks'], mode='query_toks')
            v.query_frame(extra_columns=['query'], mode='query_text')

        sim = AnseriniSimilarity(self.similarity).to_lucene_sim(self.similarity_args)
        searcher = self.index._searcher()
        searcher.object.searcher.setSimilarity(sim)

        if v.mode == 'query_lucene':
            parser = J.QueryParser("contents", searcher.object.analyzer)
            q_transform = parser.parse
            it = enumerate(inp['query_lucene'])
        elif v.mode == 'query_toks':
            parser = J.QueryParser("contents", searcher.object.analyzer)
            q_transform = _toks_query_parser_factory(parser)
            it = enumerate(inp['query_toks'])
        elif v.mode == 'query_text':
            q_transform = _noop_query_parser
            it = enumerate(inp['query'])

        if self.verbose:
            it = pt.tqdm(it, desc=str(self), total=len(inp), unit='d')

        result_cols = ['_index', 'docno', 'score', 'rank']
        if self.include_fields:
            result_cols += self.include_fields
        result = pta.DataFrameBuilder(result_cols)
        for i, query in it:
            hits = searcher.search(q_transform(query), k=self.num_results)
            records = {
                '_index': i,
                'docno': [h.docid for h in hits],
                'score': [h.score for h in hits],
                'rank': np.arange(len(hits)),
            }
            if self.include_fields:
                records.update({
                    f: [h.lucene_document.get(f) for h in hits]
                    for f in self.include_fields
                })
            result.extend(records)

        return result.to_df(merge_on_index=inp)



import os
import unittest

import pandas as pd
import pyterrier as pt

import pyterrier_anserini


class TestAnseriniRetriever(unittest.TestCase):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.index = pyterrier_anserini.AnseriniIndex.from_url(os.path.join(os.path.dirname(os.path.realpath(__file__)), "fixtures/vaswani.tar.lz4"))

    def test_vaswani(self):
        bm25 = self.index.bm25()
        qld = self.index.qld()
        tf_idf = self.index.tfidf()
        dataset = pt.get_dataset("vaswani")
        df = pt.Experiment([
                bm25,
                qld,
                tf_idf
            ],
            dataset.get_topics(),
            dataset.get_qrels(),
            ["map"])
        self.assertAlmostEqual(0.2856, df.iloc[0]["map"], places=4)
        for i in df['map']:
            self.assertGreater(i, 0)

        # check re-ranking works too
        res_in = tf_idf.search("chemical reactions")
        res_out = (tf_idf >> self.index.reranker('BM25') >> self.index.text_loader()).search("chemical reactions")
        self.assertEqual(len(res_in), len(res_out))

    @unittest.skipUnless(pyterrier_anserini.check_version('0.36.0'), "requires pyserini>=0.36.0")
    def test_vaswani_impact(self):
        impact = self.index.impact()
        res = impact(pd.DataFrame([
            {'qid': '1', 'query_toks': {'chemical': 5.3, 'reactions': 1.1}},
        ]))
        self.assertEqual(len(res), 52)
        self.assertAlmostEqual(res['score'][0], 10.6, places=4)

    @unittest.skipUnless(pyterrier_anserini.check_version('0.36.0'), "requires pyserini>=0.36.0")
    def test_include_fields(self):
        bm25_then_content = self.index.bm25() >> self.index.text_loader(['content'])
        bm25_including_content = self.index.bm25(include_fields=['content'])
        topics = pt.get_dataset('vaswani').get_topics()
        res_then_content = bm25_then_content(topics)
        res_including_content = bm25_including_content(topics)
        pd.testing.assert_frame_equal(res_then_content, res_including_content)



"""Provides functionality like indexing and retrieval from the Anserini toolkit.

This module provides a way to use the Anserini toolkit for operations like indexing
and retrieval in PyTerrier.

Typical usage example::

  import pyterrier as pt
  from pyterrier_anserini import AnseriniIndex
  index = AnseriniIndex('my_index.anserini')

  # index a dataset
  indexer = index.indexer()
  indexer.index(pt.get_dataset('vaswani').get_corpus_iter())

  # retrieve using BM25
  retr = index.bm25()
  retr(pt.get_dataset('vaswani').get_topics())
"""

__version__ = '0.1.2'

from pyterrier_anserini._java import J, set_version, check_version # noqa: I001
from pyterrier_anserini._index import AnseriniIndex
from pyterrier_anserini._indexer import AnseriniIndexer
from pyterrier_anserini._legacy import AnseriniBatchRetrieve
from pyterrier_anserini._reranker import AnseriniReRanker
from pyterrier_anserini._retriever import AnseriniRetriever
from pyterrier_anserini._text_loader import AnseriniTextLoader
from pyterrier_anserini._similarity import AnseriniSimilarity

__all__ = [
    'set_version', 'check_version', 'AnseriniIndex', 'AnseriniIndexer', 'AnseriniRetriever', 'AnseriniReRanker',
    'AnseriniBatchRetrieve', 'AnseriniSimilarity', 'AnseriniTextLoader', 'J'
]



from enum import Enum
from typing import Dict, Optional

from pyterrier_anserini import J

DEFAULT_WMODEL_ARGS = {
    'bm25.k1': 0.9,
    'bm25.b': 0.4,
    'qld.mu': 1000.,
}

class AnseriniSimilarity(Enum):
    """An enum representing the similarity functions available in Anserini."""

    bm25 = 'BM25'
    qld = 'QLD'
    tfidf = 'TFIDF'
    impact = 'Impact'

    def to_lucene_sim(self, sim_args: Optional[Dict[str, float]] = None):
        """Provides a Lucene similarity object that represents this similarity functions, including provided arguments.

        Args:
            sim_args: The arguments of this similarity function. Default values will be used when they are not provided.

        Returns:
            A ``pyjnius`` binding to a ``org.apache.lucene.search.similarities.Similarity`` object.
        """
        args = {}
        args.update(DEFAULT_WMODEL_ARGS)
        if sim_args is not None:
            args.update(sim_args)

        if self == AnseriniSimilarity.bm25:
            return J.BM25Similarity(args['bm25.k1'], args['bm25.b'])
        elif self == AnseriniSimilarity.qld:
            return J.LMDirichletSimilarity(args['qld.mu'])
        elif self == AnseriniSimilarity.tfidf:
            return J.ClassicSimilarity()
        if self == AnseriniSimilarity.impact:
            return J.ImpactSimilarity()
        raise ValueError(f"similarity {self} is not supported")

    def __repr__(self):
        return repr(self.value)



import tempfile
import unittest

import pyterrier as pt

import pyterrier_anserini


class TestAnseriniIndexer(unittest.TestCase):
    def test_index_vaswani(self):
        with tempfile.TemporaryDirectory() as d:
            index = pyterrier_anserini.AnseriniIndex(f'{d}/index')
            self.assertFalse(index.built())
            indexer = index.indexer()
            ds = pt.get_dataset('irds:vaswani')
            indexer.index(ds.get_corpus_iter())
            self.assertTrue(index.built())
            # Anything else worth asserting?
